<?php
    
namespace  Smilepay\Smilepayezcatp\Model;
use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Shipping\Model\Carrier\AbstractCarrier;
use Magento\Shipping\Model\Carrier\CarrierInterface;
use Magento\Shipping\Model\Rate\Result;


class Shippingmethod
extends \Magento\Shipping\Model\Carrier\AbstractCarrier
implements \Magento\Shipping\Model\Carrier\CarrierInterface
{
  protected $_code = 'smilepayezcatp';
  protected $smseurl = 'https://ssl.smse.com.tw/api/sppayment.asp';
  protected $smse_ezcat_url = 'https://ssl.smse.com.tw/api/ezcatGetTrackNum.asp';
  protected $table_fields = array('order_increment_id','smse_id','tracknum');
  protected $table_id_field = 'order_increment_id';

  protected $service_name = array(
                            'normal',
                            'fridge',
                            'freeze'
                        );
  protected $temperature_id = array(
                            'normal' => '0001',
                            'fridge' => '0002',
                            'freeze' => '0003'
                        );
  protected $service_max_package = array(
                            'normal'=>150,
                            'fridge'=>120,
                            'freeze'=>120
                        );

  protected $_isFixed = true;

   
  protected $_rateResultFactory;

  protected $_rateMethodFactory;
  protected $_storeManager;
  protected $_urlInterface;
  protected $_smilepayezcatp_session;
  protected $_orderFactory;
  protected $_shipmentFactory;
  protected $_trackFactory;
  protected $_trackErrorFactory;
  protected $_trackStatusFactory;
  protected $_smilepayezcatp_paymentmethod;
  protected $_smilepayezcatp_helper_data;

  public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Shipping\Model\Rate\ResultFactory $rateResultFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Sales\Model\Order $orderFactory,
        \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory,
        \Smilepay\Smilepayezcatp\Model\Session $smilepayezcatp_session,
        \Smilepay\Smilepayezcatp\Model\Paymentmethod $smilepayezcatp_paymentmethod,
        \Smilepay\Smilepayezcatp\Helper\Data   $smilepayezcatp_helper_data,
        \Magento\Sales\Model\Order\ShipmentFactory $shipmentFactory,
        \Magento\Shipping\Model\Tracking\ResultFactory $trackFactory,
        \Magento\Shipping\Model\Tracking\Result\ErrorFactory $trackErrorFactory,
        \Magento\Shipping\Model\Tracking\Result\StatusFactory $trackStatusFactory,
        array $data = []
    ) {
        $this->_rateResultFactory = $rateResultFactory;
        $this->_rateMethodFactory = $rateMethodFactory;
        $this->_storeManager=$storeManager;
        $this->_urlInterface = $urlInterface;
        $this->_orderFactory = $orderFactory;
        $this->_smilepayezcatp_session = $smilepayezcatp_session;
        $this->_smilepayezcatp_paymentmethod = $smilepayezcatp_paymentmethod;
        $this->_smilepayezcatp_helper_data =  $smilepayezcatp_helper_data;
        $this->_shipmentFactory = $shipmentFactory;
        $this->_trackFactory = $trackFactory;
        $this->_trackErrorFactory = $trackErrorFactory;
        $this->_trackStatusFactory = $trackStatusFactory;
        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);

   }
   public function getDcvc()
   {
        return trim($this->_smilepayezcatp_paymentmethod->getDcvc());
   }
   public function getVkey()
   {
        return trim($this->_smilepayezcatp_paymentmethod->getVkey());
   }
   public function getRvg2c()
   {
        return trim($this->_smilepayezcatp_paymentmethod->getRvg2c());
   }

  public function getMaxOrderTotal()
  {
    return $this->getConfigData('max_order_total');
  }
  public function getMinOrderTotal()
  {
    return $this->getConfigData('min_order_total');
  }

  public function getCurrencyLimit()
  {
    return trim($this->getConfigData('smilepay_ezcatp_currency_limit'));
  }

  public function isActive()
  {
    $result=parent::isActive();
    if( $result)
    {
        if($this->getCurrencyLimit()==true)
        {
            if(  $this->_storeManager->getStore()->getCurrentCurrencyCode() != 'TWD')
                return false;
        }

    }
    return $result;
  }
  public function collectRates(RateRequest $request)
  {
    if (!$this->isActive()) 
    {
            return false;
    }
    //check total
  
    $minTotal = $this->getConfigData('min_order_total');
    $maxTotal = $this->getConfigData('max_order_total');

    $this->_updateFreeMethodQuote($request);
    $result = $this->_rateResultFactory->create();
  
    foreach($this->service_name as $service_id)
    {
        $total = $request->getBaseSubtotalInclTax()+$this->getConfigData( $service_id.'_price');
        
        if ((!empty($minTotal) && ($total < $minTotal)) || (!empty($maxTotal) && ($total > $maxTotal))) {
               continue;
        }    
        $result->append($this->_getDefaultRate($request,$service_id));
       
    }
    return $result;
  }
 
  public function getAllowedMethods()
  {
    $method = array();
    foreach($this->service_name as $service_id)
    {
         $method[$service_id] =$this->getConfigData($service_id.'_name');
    }

    return $method;
  }
 
  protected function _getDefaultRate(RateRequest $request,$service_id)
  {
  
    if(!$this->getConfigData($service_id. '_active'))
          return null;
    $rate = $this->_rateMethodFactory->create();
    
    $rate->setCarrier($this->_code);
    $rate->setCarrierTitle($this->getConfigData('title'));
    $rate->setMethod($service_id);
    $rate->setMethodTitle($this->getConfigData($service_id.'_name'));
    $rate->setPrice($this->getConfigData($service_id.'_price'));
    $rate->setCost(0);

    //free cost over specfic total
    if ($request->getFreeShipping() 
            ||( $this->getConfigData('free_shipping_enable') == 1  
            && !is_null( $this->getConfigData('free_shipping_subtotal'))
            && !empty($this->getConfigData('free_shipping_subtotal'))
            && $request->getBaseSubtotalInclTax() >= $this->getConfigData('free_shipping_subtotal'))
       )
    {
        $rate->setCost(0);
        $rate->setPrice(0.00);
    }
    return $rate;
  }


  //Description: Connect to SMSe create ezcatp order
  //return: SMSe result array return when success,
  //return: error array when fail,

  public function runezcatpProcess($order_increment_id,$lwh)
  {
      if(!isset($order_increment_id))
      {
          $error['Status']='-1';
          $error['Desc']='Error Order id, input error!';
          return $error;
      }
      $order_id=$order_increment_id;

      $order= $this->_orderFactory->loadByIncrementId($order_id);
      if (!$order->getId()) 
      {
          $error['Status']='-2';
          $error['Desc']='Error Order id, NOT find!';
          return $error;
      }
      

      $order_result = $this->createOrderProcess($order_increment_id);

      if($order_result !=null)
      {
          if($order_result['Status']!=1)
          {
              $order_result['Desc'] = 'Process => order create<br>' .  $order_result['Desc'];
              return $order_result;
          }
         
      }



      $track_result = $this->createTracknumProcess($order_increment_id,$lwh);
      if($track_result !=null)
      {
          if($track_result['Status']!=1)
          {
              $track_result['Desc'] = 'Process => Tracknum create<br>' .  $track_result['Desc'];
              return $track_result;
          }
         
      }
      return false;
  }

  public function createOrderProcess($order_increment_id)
  {
      if(!isset($order_increment_id))
      {
          $error['Status']='-1';
          $error['Desc']='Error Order id, input error!';
          return $error;
      }
      
      $order_id=$order_increment_id;

      $order= $this->_orderFactory->loadByIncrementId($order_id);

      if (!$order->getId()) 
      {
          $error['Status']='-2';
          $error['Desc']='Error Order id, NOT find!';
          return $error;
      }
 
      $customer_name =  $order->getShippingAddress()->getLastname() . $order->getShippingAddress()->getFirstname();
      $customer_email=$order->getCustomerEmail();
      $phone = $order->getShippingAddress()->getTelephone();
      $shippingAddress=$order->getShippingAddress();
     // $address =  $shippingAddress->getData('region') . " " . $shippingAddress->getData('city') ." " .  $shippingAddress->getData('street');
      $address =   $shippingAddress->getData('city') ." " .  $shippingAddress->getData('street');
      $method_code =  $order->getShippingMethod();

      $method_arr = explode('_', $method_code);
      if(count($method_arr) > 1)
      {
          if($method_arr[0] != $this->_code)
          {
              $error['Status']='-10002';
              $error['Desc']='Error shipping method';
              return $error;
          }


      }
        
      $temperature = $this->temperature_id[$method_arr[1]];


      $saved_data=$this->getezcatpData($order_increment_id);

      if(!empty(  $saved_data))
      {
          foreach($saved_data as $data)
          {
              if(!empty( $data['smse_id']))
                return null;
          }
      }

      
      if($this->getCurrencyLimit()==true)
      {
          $total = $order->getGrandTotal();
      }
      else
      {
           
           //find TWD selected?
          if($this->_smilepayezcatp_helper_data->isCurrencyTWDselected())
              $total = $order->getGrandTotal();
          else
              $total = $order->getBaseGrandTotal();
      }
      $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
      $front_url = $objectManager->create('Magento\Framework\Url');
      $ret_url =   $front_url->getUrl("smilepayezcatp/pay/paid", array( 
                                                                '_nosid' => true 
                                    ));
        
         
      $postdata = array(
                        'Dcvc' =>trim( $this->getDcvc()),
                        'Rvg2c' => trim($this->getRvg2c()),
                        'Verify_key' =>trim($this->getVkey()),
                        'Od_sob' => $order_id,
                        'Pay_zg' => 81,
                        'Pay_subzg'=> 'TCAT',
                        'Data_id' => $order_id,
                        'Amount' => intval(round($total)),
                        'Pur_name' => $customer_name,
                        'Tel_number' =>  $phone,
                        'Mobile_number' => $phone,
                        'Address' => $address,
                        'Email' => $customer_email,
                        'temperature' => $temperature,
                        'Roturl' =>  $ret_url,
                        //'Roturl' => $this->_urlInterface->getUrl("smilepayezcatp/pay/paid"),
                        'Roturl_status'=>'magento1.8.1_Ok',
                    );
       
        //connection start
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL,$this->smseurl);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
        curl_setopt($ch,CURLOPT_POSTFIELDS,http_build_query( $postdata));
       
        $resultxml=curl_exec($ch); 
        curl_close($ch);
         

        if(!is_null( $resultxml) && !empty($resultxml) )
        {
            $xml = simplexml_load_string($resultxml);
            if (isset( $xml)) 
            {
                //(strlen(trim($xml->PayEndDate)) >8 ) ? $xmlPayEndDate=$xml->PayEndDate : $xmlPayEndDate="" ;                  
                $result= array(
                    //'PayEndDate' =>urldecode($xmlPayEndDate),
                    'Amount' => strval($xml->Amount),
                    'Data_id'=> strval($xml->Data_id),
                    'Status'=> strval($xml->Status),
                    'Desc' => strval($xml->Desc),
                    'SmilePayNO'=> strval($xml->SmilePayNO),
                 );   
    
            }
            else
            {
                 $result = array(
                            'Status' => '-1000',
                            'Desc' => '取號失敗',    
                            'Data_id'=>$order_id,
                        );     
            }
        }
        else
        {
            $result = array(
                        'Status' => '-1000',
                        'Desc'=>'取號失敗',    
                        'Data_id'=>$order_id,
                    );     
        }
        
        $Smseid = $result['SmilePayNO'];
    
        if($result['Status']==1) // success
        {
         
            
           $admincomment=sprintf(__('smilepay_ezcatp_placeorder_comment_admin_success'),$Smseid );
            //note
            $order->addStatusHistoryComment( $admincomment,false);
            /*$history=$order->addStatusHistoryComment($customercomment,false);
            $history->setIsCustomerNotified(true);
            $history->setIsVisibleOnFront(1);*/

            $order->save();
           // $order->sendOrderUpdateEmail(true, $customercomment);

            $add_data= array("order_increment_id" =>  $result['Data_id'],
                                           "smse_id" => $Smseid,
                                           );
                                           
            $this->saveezcatpData($result['Data_id'],$add_data);

        }

        return $result;
   }

   public function createTracknumProcess($order_increment_id,$lwh)
   {
      if(empty($order_increment_id))
      {
          $error['Status']='-1';
          $error['Desc']='Error Order id, input error!';
          return $error;
      }
      
      $order_id=$order_increment_id;
      $order= $this->_orderFactory->loadByIncrementId($order_id);
 

      if (!$order->getId()) 
      {
          $error['Status']='-2';
          $error['Desc']='Error Order id, NOT find!';
          return $error;
      }

      $method_code =  $order->getShippingMethod();

      $method_arr = explode('_', $method_code);
       
      $max_lwh= $this->service_max_package[$method_arr[1]];

      if($lwh>$max_lwh)
      {
          $error['Status']='-10011';
          $error['Desc']='Error LWH SIZE(Over Max Value of service.)';
          return $error;
      }

      if($lwh <= 60 )
        $packagea_size = "0001";
      elseif($lwh>60 && $lwh <= 90)
        $packagea_size = "0002";
      elseif($lwh>90 && $lwh <= 120)
        $packagea_size = "0003";
      elseif($lwh>120 && $lwh <= 150)
        $packagea_size = "0004";


     
   
      
      if(count($method_arr) > 1)
      {
          if($method_arr[0] != $this->_code)
          {
              $error['Status']='-10002';
              $error['Desc']='Error shipping method';
              return $error;
          }

      }

      $saved_data=$this->getezcatpData($order_increment_id);
  
      if(!empty(  $saved_data))
      {
          foreach($saved_data as $data)
          {
              if(empty( $data['smse_id']))
              {
                  $error['Status']='-10009';
                  $error['Desc']='Error!! smse_id DOES NOT exist.';
                  return $error;
                
              }
              elseif(!empty($data['track_num']))
                return NULL;
              $smseid=$data['smse_id'];
              break;
          }
      }
      else
      {
           $error['Status']='-10010';
           $error['Desc']='Error!! smse_id and order DOES NOT created.';
           return $error;
      }


      $postdata = array(
                        'Dcvc' =>trim( $this->getDcvc()),
                        'Rvg2c' => trim($this->getRvg2c()),
                        'Verify_key' =>trim($this->getVkey()),                  
                    );
      $postdata['smseid'] = $smseid;
      $postdata['package_size'] = $packagea_size;
      $tomorrow = mktime(0, 0, 0, date("m")  , date("d")+1, date("Y"));
      $after_tomorrow = mktime(0, 0, 0, date("m")  , date("d")+2, date("Y"));
      if(date('w',$tomorrow)==0)
        $post_data['delivery_date'] = date ('Y-m-d', $after_tomorrow);
      else
        $post_data['delivery_date'] = date ('Y-m-d',$tomorrow);
      $postdata['delivery_timezone'] = "4";
        //connection start
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL,$this->smse_ezcat_url);
      curl_setopt($ch, CURLOPT_VERBOSE, 1);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
      curl_setopt($ch, CURLOPT_POST, 1);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
      curl_setopt($ch,CURLOPT_POSTFIELDS,http_build_query( $postdata));
       
      $resultxml=curl_exec($ch); 
      curl_close($ch);


      if(!is_null( $resultxml) && !empty($resultxml) )
      {
            $xml = simplexml_load_string($resultxml);
            if (isset( $xml)) 
            {
                //(strlen(trim($xml->PayEndDate)) >8 ) ? $xmlPayEndDate=$xml->PayEndDate : $xmlPayEndDate="" ;                  
                $result= array(
                    //'PayEndDate' =>urldecode($xmlPayEndDate),
                    'TrackNum' => strval($xml->TrackNum),
                    'Amount' => strval($xml->Amount),
                    'Data_id' => strval($xml->Data_id),
                    'Status' =>strval($xml->Status),
                    'Desc' =>strval($xml->Desc)
                 );   
    
            }
            else
            {
                 $result = array(
                            'Status' => '-1000',
                            'Desc' => '取號失敗',    
                            'Data_id'=>$order_id,
                        );     
            }
      }
      else
      {
            $result = array(
                        'Status' => '-1000',
                        'Desc'=>'取號失敗',    
                        'Data_id'=>$order_id,
                    );     
      }
        
      if($result['Status']==1) // success
      {
        
           $admincomment=sprintf(__('smilepay_ezcatp_createtracknum_comment_admin_success'),$result['TrackNum'] );
            //note
            //$order->addStatusHistoryComment( $admincomment,false);
            $history=$order->addStatusHistoryComment($admincomment,false);
            $history->setIsCustomerNotified(true);
            $history->setIsVisibleOnFront(1);

           // $order->save();
           /* $order->sendOrderUpdateEmail(true, $customercomment);*/
           // $order->save();
            $add_data= array("order_increment_id" =>  $order_increment_id,
                                           "tracknum" =>$result['TrackNum']
                                           );
            $this->saveezcatpData( $order_increment_id,$add_data);
            $tracking = array(
                            array(
                                'title'=>$this->getConfigData('title'),
                                'carrier_code' =>$this->_code,
                                'number' =>$result['TrackNum']
                            )
                            
                        );
            $shipment = $this->_shipmentFactory->create($order,[], $tracking);
            
            $shipment->save();


            /*$shipment = Mage::getModel('sales/service_order', $order)->prepareShipment();
            if ($shipment) {
                $track = Mage::getModel('sales/order_shipment_track')
                    ->setNumber($result['TrackNum'])
                    ->setCarrierCode($this->_code)
                    ->setTitle($this->getConfigData('title'));
                $shipment->addTrack($track)
                    ->save();
          }*/
            
      }

      return $result;
   }

   public function saveezcatpData($order_increment_id,$data)
   {
      if(empty($order_increment_id))
        return NULL;
      if(!is_array($data))
        return NULL;
      if(!isset($data[$this->table_id_field]))
      {
          $data[$this->table_id_field] = $order_increment_id;
      }
      foreach($data as $key=>$val)
      {
         if(!in_array($key,$this->table_fields))
         {
             unset($data[$key]);
         }
      }
      $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
      $resource = $objectManager->create('Magento\Framework\App\ResourceConnection');
            
      $db_write= $resource->getConnection();
            
    


         
      $read_result = $db_write->fetchOne("select * from `". $resource->getTableName('smilepayezcatp_record'). "` where `order_increment_id` = " . $order_increment_id);
      
      if(empty($read_result))
      {
        $db_write->insert($resource->getTableName('smilepayezcatp_record'),
                                     $data
                            );
      }
      else
      {
        $whereCondition = array($this->table_id_field . '=?' => $data[$this->table_id_field]);
        $db_write->update($resource->getTableName('smilepayezcatp_record'),
                                     $data,$whereCondition
                            );
      }
      $db_write->closeConnection();
      
      return TRUE;
      
  }

  public function getezcatpData($order_increment_id)
  {
      if(empty($order_increment_id))
        return NULL;
     
      $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
      $resource = $objectManager->create('Magento\Framework\App\ResourceConnection');
      $db_read= $resource->getConnection();

      $read_result = $db_read->fetchAll("select * from `".$resource->getTableName('smilepayezcatp_record'). "` where `order_increment_id` = " . $order_increment_id);
        
      $db_read->closeConnection();
      return $read_result;

      
  }

  public function isTrackingAvailable()
  {
        return true;
  }

  public function getTracking($trackings)
  {
    $result=$this->_trackFactory->create();
    $tracking = $this->_trackStatusFactory->create();
    $tracking->setCarrier($this->_code);
    $tracking->setCarrierTitle($this->getConfigData('title'));
    $tracking->setTracking($trackings);
    $tracking->setTrackSummary( __('NOT support this tracking function'));
    
    $result->append($tracking);

         
       
      
    return $result;
  }
  public function getTrackingInfo($tracking)
  {
        $result = $this->getTracking($tracking);

        if ($result instanceof \Magento\Shipping\Model\Tracking\Result) {
            $trackings = $result->getAllTrackings();
            if ($trackings) {
                return $trackings[0];
            }
        } elseif (is_string($result) && !empty($result)) {
            return $result;
        }

        return false;
  }






}
?>